`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : ps2_ap_de0.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze ps2  system 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2014/01/21
//*****************************************************************************

module ps2_ap_de0(  
 clk_50m, resetb, 
 UART_Rx, UART_Tx,led,dip_sw,
 ps2_clk,
 ps2_dat,              
 //monitor  
 mon_ps2_clk,
 mon_ps2_data,
 mon
    );

  input  clk_50m;  //input clock 50MHz
  input  resetb;    //reset Low=reset
  input  UART_Rx;  //UART rx 115200bps
  output UART_Tx; //UART tx 115200bps
  output [3 : 0] led;   //led Hi=on
  input  [3 : 0] dip_sw; //input dip_sw
  inout ps2_clk;
  inout ps2_dat;
  output mon_ps2_clk;
  output mon_ps2_data;
  output         mon;
   
  //
  /*  
  parameter p_reg_addr_low =  32'hC0000000;
  parameter p_reg_addr_hi  =  32'hC0000007;
  parameter p_bram_addr_low = 32'hC0001000;
  parameter p_bram_addr_hi  = 32'hC00017FF;
  // 
  parameter p_wb_addr_low  =  32'hD0000000;
  parameter p_wb_addr_hi   =  32'hEFFFFFFF;
  parameter p_wb_ps2_low   =  p_wb_addr_low + 0;
  parameter p_wb_ps2_hi    =  p_wb_addr_low + 15;
  */ 
  //
  parameter p_wb_offset_low  =  32'h10000000;
  parameter p_wb_offset_hi   =  32'h2FFFFFFF;
  parameter p_wb_ps2_low   =  p_wb_offset_low + 0;
  parameter p_wb_ps2_hi    =  p_wb_offset_low + 15;
  //
  wire  clk_50m_i;  // 50MHz 
  wire  Clk; //100MHz CLk
  wire  locked;
  wire  reset_loacked;  
  wire  resetb_loacked;
  wire  Reset;
  //nios2 gpio
  wire  [31:0]   gpo;
  wire  [31:0]   gpi;
  //wishboen bus  nios2 -> IP
  wire         WB_CLK;         // clock 
  wire         WB_RST;         // reset (asynchronous active hi)
  wire         WB_CYC;         // cycle
  wire         WB_STB;         // strobe
  wire  [31:0] WB_ADR;         // address adr_i[1]
  wire         WB_WE;          // write enable
  wire  [31:0] WB_DAT_I;          // data output
  wire  [31:0] WB_DAT_O;          // data input
  wire  [3:0]  WB_SEL;         // byte enable
  wire         WB_ACK;         // normal bus termination
  wire         WB_ERR;         // err bus termination
  wire         WB_RTY;         // rtry
  // 
  //ps2
  wire         cyc_i_ps2;         // cycle
  wire         stb_i_ps2;         // strobe
  wire         we_i_ps2;          // write enable
  wire  [15:0] adr_i_ps2;         // address adr_i
  wire  [7:0] dat_o_ps2;   // data input
  wire         ack_o_ps2;         // normal bus termination
  wire         wb_ps2_sel;
  wire         irq_o_ps2;
   
   
  reg [23:0] clk_cnt;
  reg ps2_clk_d1;
  reg ps2_clk_d2;
  reg ps2_clk_d3;
  reg mon_ps2_clk;
  reg ps2_data_d1;
  reg ps2_data_d2;
  reg ps2_data_d3;
  reg mon_ps2_data;
//mon
  reg         mon;



  //reset 
  assign reset = ~resetb;
  assign reset_locked =((resetb==1'b0)||(locked==1'b0))?1'b1:1'b0;
  assign Reset = reset_locked ;
  assign resetb_locked = ~reset_locked ;
     
  //led
  assign led = {gpo[3:0]};

  assign gpi = {28'h0000000,dip_sw};


clk_gen clk_gen(
   .areset(reset),   
   .inclk0(clk_50m),
   .c0(clk_50m_i),   // 50MHz
   .c1(Clk),  //100MHz
   .locked(locked));
   
nios2e nios2e(
   .reset_reset_n(resetb_locked),
   .clk_clk(Clk),
   .uart_0_txd(UART_Tx),    
   .uart_0_rxd(UART_Rx),
   .gpo_export(gpo),
   .gpi_export(gpi),
  //wishbone nios2 -> IP
   .wb_CLK_O(WB_CLK),         // clock
   .wb_RST_O(WB_RST),         // reset 
   .wb_CYC_O(WB_CYC),         // cycle
   .wb_STB_O(WB_STB),         // strobe
   .wb_ADR_O(WB_ADR),         // address adr
   .wb_SEL_O(WB_SEL),         // byte_sel
   .wb_WE_O( WB_WE),          // write enable
   .wb_DAT_O(WB_DAT_I),       // data output
   .wb_DAT_I(WB_DAT_O),       // data input
   .wb_ACK_I(WB_ACK),         // normal bus termination
   .wb_ERR_I(WB_ERR),         // error bus termination
   .wb_RTY_I(WB_RTY)          // retry
        );
   
//opencores ps2core
 ps2_wb ps2_wb_0(
  .wb_clk_i(WB_CLK),       // clock
  .wb_rst_i(WB_RST),       // reset 
  .wb_stb_i(stb_i_ps2),    // strobe
  .wb_adr_i(adr_i_ps2[0]), // address adr_i
  .wb_we_i (we_i_ps2),      // write enable
  .wb_dat_i(WB_DAT_I[7:0]),// data output
  .wb_dat_o(dat_o_ps2),    // data input
  .wb_ack_o(ack_o_ps2),    // normal bus termination
  .irq_o   (irq_o_ps2),
  .ps2_clk (ps2_clk),
  .ps2_dat (ps2_dat)); 
   
//WihsBoneBus 
   assign wb_ps2_sel = ((WB_ADR>=p_wb_ps2_low)&&(WB_ADR<=p_wb_ps2_hi))? 1'b1:1'b0;
   assign cyc_i_ps2  =(wb_ps2_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_ps2  =(wb_ps2_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_ps2  ={12'h000,WB_ADR[3:0]};
   assign we_i_ps2   =(wb_ps2_sel==1'b1)?WB_WE:1'b0;   
   assign WB_DAT_O   =(wb_ps2_sel==1'b1)?{dat_o_ps2,dat_o_ps2,dat_o_ps2,dat_o_ps2}:
                                        32'h00000000;   
   assign WB_ACK = (wb_ps2_sel==1'b1)?ack_o_ps2:WB_STB;
   assign WB_ERR = 1'b0;
   assign WB_RTY = 1'b0;
   
always @ (posedge Clk )
  if (Reset==1'b1)
     clk_cnt <= 24'd0;
  else
     clk_cnt <= clk_cnt + 24'd1;
  
//mon
   
always @ (posedge Clk )
    begin
      ps2_clk_d1 <=  ps2_clk;
      ps2_clk_d2 <=  ps2_clk_d1;
      ps2_clk_d3 <=  ps2_clk_d2;
      mon_ps2_clk <=  ps2_clk_d3;
      ps2_data_d1 <=  ps2_dat;
      ps2_data_d2 <=  ps2_data_d1;
      ps2_data_d3 <=  ps2_data_d2;
      mon_ps2_data <=  ps2_data_d3;
    end // always @ (posedge clk )
   
always @ (posedge clk_cnt[6] )
  mon <= ^dat_o_ps2;
endmodule
